#include "generic.h"
// #include "getopt.h"
// #include <sys/timeb.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <math.h>

double max_shade = -1;

double sqr(double v)
{
  return v*v;
} // sqr

inline void SetPixelColor(SDL_Surface *screen, int x, int y, unsigned char red, unsigned char green, unsigned char blue)
{
  ((unsigned char*)(screen->pixels))[x*3+0 + y*screen->pitch] = red;
  ((unsigned char*)(screen->pixels))[x*3+1 + y*screen->pitch] = green;
  ((unsigned char*)(screen->pixels))[x*3+2 + y*screen->pitch] = blue;
} // SetPixelColor

void SetShade(SDL_Surface *screen, int x, int y, double shade)
{
  int value = 255-((int)((fabs(shade)/1.5)*255.0));

  SetPixelColor(screen, x, y, value, value, value);
} // SetShade

inline void SetReplicate(SDL_Surface* screen, int x, int y, double dist, int circle_x, int circle_y)
{
  SetShade(screen, x+circle_x, y+circle_y, dist);
  SetShade(screen, circle_x-x, y+circle_y, dist);

  SetShade(screen, circle_x+y, circle_y+x, dist);
  SetShade(screen, circle_x-y, circle_y+x, dist);

  SetShade(screen, x+circle_x, circle_y-y, dist);
  SetShade(screen, circle_x-x, circle_y-y, dist);

  SetShade(screen, circle_x+y, circle_y-x, dist);
  SetShade(screen, circle_x-y, circle_y-x, dist);
} // SetPixel_

inline void swap(int &a, int &b)
{
  int t;

  t = a;
  a = b;
  b = t;
} // swap

int main(int argc, char** argv)
{
    SDL_Surface      *screen, *backup_screen;
    SDL_Event        event;
    SDL_Rect         dst_rect, full_image_rect, scrolled_rect, location;
    int              window_width, window_height, thumb_height, thumb_border_size;
    long             pixel_set = 0;
    bool             l_button_down = false;

    window_width = 512; window_height = 512;

    if(SDL_Init(SDL_INIT_VIDEO) < 0) {
	cerr << "Could not initialize SDL video" << endl;
	exit(-1);
    } // if
    atexit(SDL_Quit);

    Uint32 flags = 0;

    // flags |= SDL_FULLSCREEN;

    screen = SDL_SetVideoMode(window_width, window_height, 24, flags);

    if(!screen) {
	cerr << "could not SetVideoMode() - abort" << endl;
	exit(-1);
    } // if

    int    x, y, a, b, radius;
    int    circle_x, circle_y;
    double dist, shade, subst, add;

    radius = 200;

    circle_x = 250; circle_y = 250;

    for(int repeat = 0; repeat < 1000; repeat++) {

      bzero(screen->pixels, window_width*window_height*3);

      y = radius;
      
      dist = 1.25 - radius;
      
      // first region

      subst = sqr(radius);

      x = 0;

      subst = sqr(x) + sqr(y);
      
      for(; x <= cos(M_PI/4.0)*radius+1; x++) {
	
	shade = sqrt(subst+2*y+1) - radius;
	SetReplicate(screen, x, y+1, shade, circle_x, circle_y);
	shade = sqrt(subst) - radius; 
	SetReplicate(screen, x, y, shade, circle_x, circle_y);
	shade = sqrt(subst-2*y+1) - radius; 
	SetReplicate(screen, x, y-1, shade, circle_x, circle_y);

	SetShade(screen, circle_x+x+15, circle_y+y+15, 0.0);

       	if(dist > 0) { // choose SE	
	  add = 2*(x-y)+1;
	  dist += add+4;
	  subst += add;
	  y--;
	} // if
	else {         // choose S
	  add = 2*x+1;
	  dist += add+2;
	  subst += add;
	} // else
      } // for
      
      SDL_UnlockSurface(screen);
      SDL_UpdateRect(screen, 0, 0, window_width, window_height);
      SDL_LockSurface(screen);
      
      for(;;) {
	
	if((SDL_WaitEvent(&event) == 0) && (event.type == SDL_KEYDOWN)) 
	  goto exit_app;
	
	switch(event.type) {
	  
	case SDL_MOUSEMOTION:	      
	  if(l_button_down) {
	    radius = sqrt(sqr(circle_x - event.motion.x) + sqr(circle_y - event.motion.y));
	    if(radius > window_width/2-10) radius = window_width/2-10;
	    goto cont_app;
	  } // if
	  break;
	  
	case SDL_MOUSEBUTTONDOWN:
	  l_button_down = true;
	  break;
	  
	case SDL_MOUSEBUTTONUP:
	  l_button_down = false;
	  break;
	  
	case SDL_KEYDOWN:
	  
	  switch(event.key.keysym.sym) {
	    
	  case '+':
	    radius += 3;
	    goto cont_app;
	    break;

	  case '-':
	    radius -= 3;
	    goto cont_app;
	    break;

	  case 27:
	  case 'q':
	  case 'Q':
	    goto exit_app;
	    
	  default:
	    break;
	  } // switch
	} // switch
	
      } // for

    cont_app:
      ;
    } // for

 exit_app:

    SDL_UnlockSurface(screen);	    
    SDL_FreeSurface(screen);

    exit(0);
} // main
